<?php

namespace MDT\Apple_News_Core_Enhancements;

/**
 * Class Admin
 *
 * @package MDT\Apple_News_Core_Enhancements
 */
class Admin {

	/**
	 * Meta key for section names
	 *
	 * @var string
	 */
	const META_KEY_SECTION_NAMES = 'apple_news_api_section_names';

	/**
	 * Meta key for errors
	 *
	 * @var string
	 */
	const META_KEY_ERRORS = 'mdt_apple_news_api_errors';

	/**
	 * Admin constructor.
	 */
	public static function init() {
		add_filter( 'manage_posts_columns', array( __CLASS__, 'register_custom_manage_posts_column' ), 10, 2 );
		add_action( 'manage_posts_custom_column', array( __CLASS__, 'custom_manage_posts_content' ), 10, 2 );

		add_action( 'init', array( __CLASS__, 'register_meta_fields' ), 15 );

		// Save some additional post meta from the push result
		add_action( 'apple_news_before_push', array( __CLASS__, 'add_push_actions' ) );

		// Save some additional post meta from the push result
		add_action( 'apple_news_after_push', array( __CLASS__, 'save_details' ), 5, 2 );
	}

	/**
	 * Whether or not to output the post columns
	 *
	 * @return bool
	 */
	public static function output_columns() {
		return apply_filters( 'mdt_apple_news_ce_output_columns', true );
	}

	/**
	 * Adds the Apple News Status column to the manage posts view
	 *
	 * @param array  $defaults  The default post columns
	 * @param string $post_type The post type being managed
	 * @return mixed The filtered post columns
	 */
	public static function register_custom_manage_posts_column( $defaults, $post_type ) {
		if (
			self::output_columns()
			&& in_array( $post_type, self::get_allowed_post_types(), true )
			&& self::is_valid_user()
		) {
			$defaults['apple_news_status'] = 'Apple News Status';
		}

		return $defaults;
	}

	/**
	 * Echoes out the apple news status as returned from the plugin itself
	 *
	 * @param string $column_name The name of the column
	 * @param int    $post_id     The post ID
	 */
	public static function custom_manage_posts_content( $column_name, $post_id ) {
		if (
			self::output_columns()
			&& class_exists( '\Admin_Apple_News' )
			&& 'apple_news_status' === $column_name
		) {
			echo esc_html( \Admin_Apple_News::get_post_status( $post_id ) );
		}
	}

	/**
	 * Is the user an author or higher
	 *
	 * @return bool
	 */
	public static function is_valid_user() {

		$capability = apply_filters( 'mdt_apple_news_ce_columns_capability', 'edit_published_posts' );

		if ( ! current_user_can( $capability ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Register meta fields for our details meta box
	 */
	public static function register_meta_fields() {

		// Register the section names meta field
		if( !function_exists( '\apple_news_register_meta_helper' ) ) {
			return;
		}

		\apple_news_register_meta_helper(
			'post',
			self::get_allowed_post_types(),
			self::META_KEY_SECTION_NAMES,
			array(
				'type'         => 'array',
				'show_in_rest' => array(
					'schema' => array(
						'type'  => 'array',
						'items' => array(
							'type' => 'string',
						),
					),
				),
			)
		);

		// Don't show errors in REST unless user is logged in
		$show_errors_in_rest = false;

		if ( is_user_logged_in() ) {

			// Define the error object schema
			$show_errors_in_rest = array(
				'schema' => array(
					'type'  => 'array',
					'items' => array(),
				),
			);
		}

		// Register the errors meta field
		\apple_news_register_meta_helper(
			'post',
			self::get_allowed_post_types(),
			self::META_KEY_ERRORS,
			array(
				'type'         => 'array',
				'show_in_rest' => $show_errors_in_rest,
			)
		);
	}

	/**
	 * Add an action to check the push response
	 *
	 * @param int $post_id The post ID
	 */
	public static function add_push_actions( $post_id ) {

		/**
		 * Check push request and see if any errors were returned
		 *
		 * @param object $response The response object
		 * @param string $context Context under which the hook is fired.
		 * @param string $class HTTP transport used.
		 * @param array $parsed_args HTTP request arguments.
		 * @param string $url The request URL.
		 * 
		 */
		add_action(
			'http_api_debug',
			function( $response, $context, $class, $parsed_args, $url ) use ( $post_id ) {

				// Check for bad requests
				// Skip if this isn't an Apple News api request.
				if ( 0 !== strpos( $url, 'https://news-api.apple.com' ) ) {
					return;
				}

				// If the whole request is erred, log the wp_error messages
				if( is_wp_error($response) ) {
					update_post_meta( $post_id, self::META_KEY_ERRORS, $response->get_error_messages() );
					return;
				}

				// If the response is okay, return
				if(
					empty( $response['response']['code'] )
					|| 400 !== $response['response']['code']
					|| empty( $response['body'] )
				){
					return;
				}

				// Look for an errors property and log in post meta if set
				$body = json_decode( $response['body'] );
				if ( ! empty( $body->errors ) ) {
					update_post_meta( $post_id, self::META_KEY_ERRORS, $body->errors );
				}
			}, 10, 5
		);
	}

	/**
	 * Save some info from the push
	 *
	 * @param int    $post_id The post ID
	 * @param object $result  The result of the push
	 */
	public static function save_details( $post_id, $result ) {

		// Remove this hook so we're not running http_api_debug action here
		remove_action( 'apple_news_before_push', array( __CLASS__, 'add_push_actions' ) );

		// Get post sections, and the array of section objects
		$post_sections   = \Admin_Apple_Sections::get_sections_for_post( $post_id );
		$section_objects = \Admin_Apple_Sections::get_sections();

		// Store section names
		$section_names = array();

		foreach ( $post_sections as $section_url ) {
			foreach ( $section_objects as $section_object ) {
				if ( ! empty( $section_object->name )
					&& isset( $section_object->links->self )
					&& $section_object->links->self === $section_url
				) {
					$section_names[] = $section_object->name;
				}
			}
		}

		// Save section names in post meta
		if ( ! empty( $section_names ) ) {
			update_post_meta( $post_id, self::META_KEY_SECTION_NAMES, $section_names );
		}

		// Re-add the http_api_debug action
		add_action( 'apple_news_before_push', array( __CLASS__, 'add_push_actions' ) );
	}

	/**
	 * Get the array of allowed post types
	 *
	 * @return array The array of allowed post types
	 */
	public static function get_allowed_post_types() {

		if ( ! class_exists( '\Admin_Apple_Settings' ) ) {
			return array();
		}

		$admin_settings = new \Admin_Apple_Settings();
		$settings       = $admin_settings->fetch_settings();

		/**
		 * Filter the post types that we provide enhancements for
		 *
		 * @param array The existing post types as defined by the parent plugin
		 */
		return apply_filters( 'mdt_apple_news_core_enhancements_post_types', $settings->get( 'post_types' ) );
	}
}

new Admin();
