/**
 * WordPress dependencies
 */
import { PluginDocumentSettingPanel } from '@wordpress/edit-post'
import { select, withSelect } from '@wordpress/data'
import { compose, withInstanceId } from '@wordpress/compose'
import { Disabled, TextControl } from '@wordpress/components'

/**
 * Returns the current version of the publish_to_apple_news rest field
 *
 * @returns {boolean}
 */
const getDetails = () => {
	const { getEditedPostAttribute } = select( 'core/editor' )

	const meta = getEditedPostAttribute( 'meta' )
	const shareUrl = meta?.apple_news_api_share_url ?? ''
	const sectionNames = meta?.apple_news_api_section_names ?? ''
	const errors = meta?.mdt_apple_news_api_errors ?? ''

	return {
		shareUrl,
		sectionNames,
		errors
	}
}

/**
 * Output a single section
 *
 * @param sectionName string
 * @returns {*} JSX
 */
const outputSection = (sectionName) => {
	return (
		<li>{ sectionName }</li>
	);
}

/**
 * Output a single error
 *
 * @param error object
 * @returns {*} JSX
 */
const outputError = (error) => {
	return (
		<div className="apple-news-details__error">
			{ error?.code &&
				<strong>
					{error.code}
				</strong>
			}
			{ error?.message && 
				<div className="apple-news-details__error-message">
					{error.message}
				</div>
			}
			{ typeof(error) === 'string' &&
				<div className="apple-news-details__error-message">
					{error}
				</div>
			}
		</div>
	);
}

/**
 * The JSX that displays the 'Apple News Details' checkbox
 *
 * @param details object
 * @param onChange function
 * @returns {*} JSX
 */
const metaBoxOutput = ( {details, postStatus} ) => {

	// Don't output this panel unless the post is published
	// (i.e. pushed to Apple News)
	if (postStatus !== 'publish') {
		return null;
	}

	return(
		<PluginDocumentSettingPanel 
			title="Apple News Details"
			className="apple-news-details"
		>
			<div className="apple-news-details__field">
				{ details.shareUrl !== '' &&
					<>
						<span className="apple-news-details__share-url">
							<a href={ details.shareUrl } target="_blank">Share URL</a>
						</span>
						<Disabled>
							<TextControl
								label=""
								value={ details.shareUrl }
							/>
						</Disabled>
					</>
				}
				{ details.shareUrl === '' &&
					<em>No Share URL set</em>
				}
			</div>
			<hr />
			<div className="apple-news-details__field">
				<div className="apple-news-details__label">Sections</div>
				<ul className="apple-news-details__sections">
					{ details?.sectionNames &&
						details.sectionNames.map( outputSection )
					}
					{ ! details?.sectionNames || details.sectionNames.length === 0 &&
						<em>No sections found</em>
					}
				</ul>
			</div>
			{ details.shareUrl === '' &&
				<>
					<hr />
					<div className="apple-news-details__field">
						<div className="apple-news-details__label">Errors</div>
						<div className="apple-news-details__errors">
							{ details?.errors &&
								details.errors.map( outputError )
							}
							{ ! details?.errors || details.errors.length === 0 &&
								<p><em>No errors found</em></p>
							}
						</div>
					</div>
				</>
			}
		</PluginDocumentSettingPanel>
	)
}

/**
 * Wrapping our output method in the needed select/dispatch higher order components
 */
export default compose(
	withSelect( (select) => {
		return {
			details: getDetails(),
			postStatus: select( 'core/editor' ).getEditedPostAttribute( 'status' )
		};
	} ),
	withInstanceId,
)( metaBoxOutput );
